#include <Bounce.h>

static const uint8_t nButtons = 17;
//associate each button with a pin number, using an array with logical names
enum buttons { a, b, x, y, l, r, zl, zr, al, ar, tl, tr, st, sel, emu, keyt, stahp};
//keyt and stahp should both be a toggle, not a momentary, switch

//button list declared as bounce entities in order to use the debounce function
Bounce buttonList[nButtons] {
  Bounce(a, 10),      //pin 0
  Bounce(b, 10),      //pin 1
  Bounce(x, 10),      //pin 2
  Bounce(y, 10),      //pin 3
  Bounce(l, 10),      //pin 4
  Bounce(r, 10),      //pin 5
  Bounce(zl, 10),     //pin 6
  Bounce(zr, 10),     //pin 7
  Bounce(al, 10),     //pin 8
  Bounce(ar, 10),     //pin 9
  Bounce(tl, 10),     //pin 10
  Bounce(tr, 10),     //pin 11
  Bounce(st, 10),     //pin 12
  Bounce(sel, 10),    //pin 13
  Bounce(emu, 10),    //pin 14
  Bounce(keyt, 10),   //pin 15
  Bounce(stahp, 10),  //pin 16
};

//joystick & virtual d-pad & mouse axis reading values
//when using 90 degree rotated 3ds sliders, connect oppisite stated axis
int yValue;     //pin 17 A3
int xValue;     //pin 18 A4
int cyValue;    //pin 19 A5
int cxValue;    //pin 20 A6
int dpadY;      //pin 21 A7
int dpadX;      //pin 22 A8
int mouseY;     //pin 23 A9
int mouseX;     //pin 24 A10

//slider deadzones & range & mapping values
#define dedcntr 512
#define posded 612
#define negded 412
#define sliderpos 850
#define sliderneg 130
#define mappos 1004
#define mapneg 20

//mouse & virtual d-pad setup
#define digposded 740
#define dignegded 284
#define up 18
#define down 19
#define left 20
#define right 21
#define mousemove 1
int xmove;
int ymove;

//rumble feedback setup
unsigned int rumble = 0;

//other setup
unsigned int i = 0;


void setup() {
  //manually control when gamepad data is sent
  Joystick.useManualSend(true);
  for (int i = 0; i < nButtons; i++) {
    //pins configured as pulldown resistors (signal is active HIGH) so the Capacitive Switches can output ~3.3V for an 'on' signal (The TeensyLC is NOT 5V tolerant)
    pinMode(i, INPUT_PULLDOWN);
  }
  pinMode(25, OUTPUT);
  digitalWrite(25, HIGH);
}


void loop() {
  //joystick logic start ----------------------------------------------
  yValue = map(analogRead(3), sliderneg, sliderpos, mapneg, mappos);
  if (yValue > posded || yValue < negded)
    Joystick.Y(yValue);
  else
    Joystick.Y(dedcntr);

  xValue = map(analogRead(4), sliderneg, sliderpos, mapneg, mappos);
  if (xValue > posded || xValue < negded)
    Joystick.X(xValue);
  else
    Joystick.X(dedcntr);

  cyValue = map(analogRead(5), sliderneg, sliderpos, mapneg, mappos);
  if (cyValue > posded || cyValue < negded)
    Joystick.Z(cyValue);
  else
    Joystick.Z(dedcntr);

  cxValue = map(analogRead(6), sliderneg, sliderpos, mapneg, mappos);
  if (cxValue > posded || cxValue < negded)
    Joystick.Zrotate(cxValue);
  else
    Joystick.Zrotate(dedcntr);


  for (uint8_t i = 1; i <= nButtons; i++) {
    //if there has been a change in the state of this button
    if (buttonList[i - 1].update()) {
      //if the button is being pressed
      if (buttonList[i - 1].risingEdge()) {
        Joystick.button(i, 1);                 //set the button as pressed
//        rumble = 1;
      }
      else {
        Joystick.button(i, 0);                 //else set it as released
      }
    }
  }

//  if (rumble == 1)
//  {
//    //make the rumble feedback occur
//    digitalWrite(25, LOW);
//    //don't delay too much or the button sample rate will be too low for basic operation
//    delay(100);
//    digitalWrite(25, HIGH);
//    delay(100);
//    //reset the rumble tag for the next check
//    rumble = 0;
//  }
  digitalWrite(25, HIGH);

  if ((buttonList[stahp].read() == LOW) && (buttonList[keyt].read() == LOW))
  {
    //send gamepad data
    Joystick.send_now();
  }
  //joystick logic end ----------------------------------------------

  //virtual d-pad logic start ---------------------------------------
  dpadY = map(analogRead(7), sliderneg, sliderpos, mapneg, mappos);
  if (buttonList[keyt].read() == LOW) {
    if (dpadY > digposded) {
      Joystick.button(up, 1);
      Joystick.button(down, 0);
    }
    else if (dpadY < dignegded) {
      Joystick.button(up, 0);
      Joystick.button(down, 1);
    }
    else {
      Joystick.button(up, 0);
      Joystick.button(down, 0);
    }
  }

  else if (buttonList[keyt].read() == HIGH) {
    if (dpadY > digposded) {
      Keyboard.set_key1(0);
      Keyboard.set_key1(KEY_UP);
    }
    else if (dpadY < dignegded) {
      Keyboard.set_key1(0);
      Keyboard.set_key1(KEY_DOWN);
    }
    else {
      Keyboard.set_key1(0);
    }
  }

  dpadX = map(analogRead(8), sliderneg, sliderpos, mapneg, mappos);
  if (buttonList[keyt].read() == LOW) {
    if (dpadX > digposded) {
      Joystick.button(left, 1);
      Joystick.button(right, 0);
    }
    else if (dpadX < dignegded) {
      Joystick.button(left, 0);
      Joystick.button(right, 1);
    }
    else {
      Joystick.button(left, 0);
      Joystick.button(right, 0);
    }
  }

  else if (buttonList[keyt].read() == HIGH) {
    if (dpadX > digposded) {
      Keyboard.set_key1(0);
      Keyboard.set_key1(KEY_LEFT);
    }
    else if (dpadX < dignegded) {
      Keyboard.set_key1(0);
      Keyboard.set_key1(KEY_RIGHT);
    }
    else {
      Keyboard.set_key1(0);
    }
  }
  //virtaul d-pad logic end ----------------------------------------

  //mouse logic start ----------------------------------------------
  mouseY = map(analogRead(9), sliderneg, sliderpos, mapneg, mappos);
  if (mouseY > digposded)
    ymove = -mousemove;
  else if (mouseY < dignegded)
    ymove = mousemove;
  else
    ymove = 0;

  mouseX = map(analogRead(10), sliderneg, sliderpos, mapneg, mappos);
  if (mouseX > digposded)
    xmove = mousemove;
  else if (mouseX < dignegded)
    xmove = -mousemove;
  else
    xmove = 0;

  if (buttonList[stahp].read() == LOW)
  {
    Mouse.move(xmove, ymove);
  }
  //mouse logic end ----------------------------------------------

  //keyboard logic start ----------------------------------------------
  if ((buttonList[stahp].read() == LOW) && (buttonList[keyt].read() == HIGH)) {
    if (buttonList[a].risingEdge())
      Keyboard.set_key1(KEY_D);
    else if (buttonList[a].fallingEdge())
      Keyboard.set_key1(0);

    if (buttonList[b].risingEdge())
      Keyboard.set_key1(KEY_S);
    else if (buttonList[b].fallingEdge())
      Keyboard.set_key1(0);

    if (buttonList[x].risingEdge())
      Keyboard.set_key1(KEY_W);
    else if (buttonList[x].fallingEdge())
      Keyboard.set_key1(0);

    if (buttonList[y].risingEdge())
      Keyboard.set_key1(KEY_A);
    else if (buttonList[y].fallingEdge())
      Keyboard.set_key1(0);

    if (buttonList[l].risingEdge())
      Mouse.set_buttons(1, 0, 0);
    else if (buttonList[l].fallingEdge())
      Mouse.set_buttons(0, 0, 0);

    if (buttonList[r].risingEdge())
      Mouse.set_buttons(0, 0, 1);
    else if (buttonList[r].fallingEdge())
      Mouse.set_buttons(0, 0, 0);

    if (buttonList[zl].risingEdge())
      Mouse.scroll(1);
    else if (buttonList[zl].fallingEdge())
      Mouse.scroll(0);

    if (buttonList[zr].risingEdge())
      Mouse.scroll(-1);
    else if (buttonList[zr].fallingEdge())
      Mouse.scroll(0);

    if (buttonList[al].risingEdge())
      Mouse.set_buttons(0, 1, 0);
    else if (buttonList[al].fallingEdge())
      Mouse.set_buttons(0, 0, 0);

    if (buttonList[ar].risingEdge())
      Keyboard.set_key1(KEY_SPACE);
    else if (buttonList[ar].fallingEdge())
      Keyboard.set_key1(0);

    if (buttonList[tl].risingEdge())
      Keyboard.set_media(KEY_MEDIA_VOLUME_INC);
    else if (buttonList[tl].fallingEdge())
      Keyboard.set_media(0);

    if (buttonList[tr].risingEdge())
      Keyboard.set_media(KEY_MEDIA_VOLUME_DEC);
    else if (buttonList[tr].fallingEdge())
      Keyboard.set_media(0);

    if (buttonList[st].risingEdge())
      Keyboard.set_key1(KEY_ENTER);
    else if (buttonList[st].fallingEdge())
      Keyboard.set_key1(0);

    if (buttonList[sel].risingEdge())
      Keyboard.set_key1(KEY_TAB);
    else if (buttonList[sel].fallingEdge())
      Keyboard.set_key1(0);

    if (buttonList[emu].risingEdge())
      Keyboard.set_key1(KEY_ESC);
    else if (buttonList[emu].fallingEdge())
      Keyboard.set_key1(0);
  }
  Keyboard.send_now();
  //keyboard logic end ----------------------------------------------
}
